"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const hrtime_now_1 = require("hrtime-now");
const ts_simple_ast_1 = require("ts-simple-ast");
const ts = __importStar(require("typescript"));
const typescript_ast_util_1 = require("typescript-ast-util");
/**
# Description

Suggests declaring an empty class or interface when a something is extending or implementing something that is not found

# Example

```
class Onion extends NonExistent implements NonExistentInterface{}
```

# Attacks
 
    "code": "2304",	"message": "Cannot find name 'NonExistentInterface'.",

# TODO
 
 * issue: move scroll to end of file
 * issue - break  target class' jsdoc
 * modify the file using  insertClass or insertInterface using Structures no text
 * var a = nonexistentFunction(1) <--- is suggested here and should not
 * config

 */
exports.declareClass = {
    name: 'declareClass',
    config: {
        // TODO: declare it in new file ? only if user put a special comment ? 
        inNewFile: false,
        // TODO: could be true|false|string . add jsdoc to new class/interface declaration
        jsdoc: true,
    },
    predicate: (arg) => {
        if (ts.isIdentifier(arg.containingTargetLight) &&
            !ts.isTypeReferenceNode(arg.containingTargetLight.parent) &&
            !ts.isCallExpression(arg.containingTargetLight.parent) &&
            !ts.isBinaryExpression(arg.containingTargetLight.parent) &&
            (arg.diagnostics.find(d => d.code === 2304 && d.start === arg.containingTargetLight.getStart()) ||
                (arg.containingTargetLight.parent.kind === ts.SyntaxKind.NewExpression &&
                    arg.diagnostics.find(d => d.code === 2552 && d.start === arg.containingTargetLight.getStart())))) {
            return true;
        }
        else {
            arg.log('predicate false because child.kind dont match ' + typescript_ast_util_1.getKindName(arg.containingTargetLight.kind));
            return false;
        }
    },
    description: (arg) => {
        const h = typescript_ast_util_1.findAscendant(arg.containingTargetLight, ts.isHeritageClause);
        let decl = typescript_ast_util_1.findAscendant(arg.containingTargetLight, n => ts.isInterfaceDeclaration(n) || ts.isClassDeclaration(n));
        const what = h && decl && (ts.isInterfaceDeclaration(decl) || h.getText().includes('implements') ? 'interface' : 'class') || (arg.containingTargetLight.parent.kind === ts.SyntaxKind.NewExpression ? 'class' : undefined);
        return `Declare ${what || 'type'} "${arg.containingTargetLight.getText()}"`;
    },
    apply: (arg) => {
        const t0 = hrtime_now_1.now();
        let simpleClassDec = arg.simpleNode.getFirstAncestorByKind(ts.SyntaxKind.ClassDeclaration);
        if (!simpleClassDec) {
            simpleClassDec = arg.simpleNode.getFirstAncestorByKind(ts.SyntaxKind.InterfaceDeclaration);
        }
        let what;
        let code;
        let start = arg.simpleNode.getFirstAncestor(a => ts_simple_ast_1.TypeGuards.isBlock(a.getParent()) || ts_simple_ast_1.TypeGuards.isSourceFile(a.getParent())).getFullStart();
        if (simpleClassDec) {
            let h = arg.simpleNode.getFirstAncestorByKind(ts.SyntaxKind.HeritageClause);
            if (!ts_simple_ast_1.TypeGuards.isHeritageClause(h)) {
                return;
            }
            what = simpleClassDec.getKind() === ts.SyntaxKind.InterfaceDeclaration ? 'interface' : h.getToken() === ts.SyntaxKind.ImplementsKeyword ? 'interface' : 'class';
            code =
                `
${simpleClassDec.isExported() ? 'export ' : ''}${what} ${arg.simpleNode.getText()} {

}
`;
        }
        else if (ts_simple_ast_1.TypeGuards.isNewExpression(arg.simpleNode.getParent())) {
            what = 'class';
            code =
                `${what} ${arg.simpleNode.getText()} {

}
`;
        }
        else {
            arg.log('not applying cause cannot find any ancestor of kind ClassDeclaration|InterfaceDeclaration nor its parent is a newexpression');
        }
        return {
            edits: [
                {
                    fileName: arg.sourceFile.fileName,
                    textChanges: [
                        {
                            newText: '\n' + code,
                            span: {
                                start,
                                length: 0
                            }
                        }
                    ]
                }
            ]
        };
    }
};
//# sourceMappingURL=declareClass.js.map