"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const hrtime_now_1 = require("hrtime-now");
const ts_simple_ast_1 = require("ts-simple-ast");
const ts = __importStar(require("typescript"));
const typescript_ast_util_1 = require("typescript-ast-util");
const util_1 = require("../util");
let newExpr;
/**
# description

adds missing constructor

# attacks

"code": "2554","message": "Expected 0 arguments, but got 3.",

# example

```alpha2 = new Alpha('hello', 1, new Date())```

# TODO

 * should call super if class extends
 * config
 * Issue (non critical): new Gamma('hello', 1, new Date())  if you stand at `Date` then it will suggest "create constructor Date"
 
 */
exports.codeFixCreateConstructor = {
    name: 'Declare constructor',
    config: {
        // TODO
        variableType: 'const',
        // TODO 'none'|'private'|'public'|'protected' 
        constructorParameterScope: 'none',
        // TODO could be false|true|string
        jsdoc: true
    },
    predicate: (arg) => {
        newExpr = ts.isNewExpression(arg.containingTarget) ? arg.containingTarget : typescript_ast_util_1.findAscendant(arg.containingTarget, ts.isNewExpression);
        if (newExpr && arg.diagnostics.find(d => d.code === 2554 && d.start <= arg.containingTargetLight.getStart() && d.start + d.length >= arg.containingTargetLight.getEnd())) {
            return true;
        }
        else {
            arg.log(`predicate false because no NewExpression ascendant was   und containingTarget.kind==${typescript_ast_util_1.getKindName(arg.containingTarget.kind)}, containingTarget.parent.kind==${arg.containingTarget.parent && typescript_ast_util_1.getKindName(arg.containingTarget.parent.kind)}`);
            return false;
        }
    },
    description: (arg) => `Declare constructor "${newExpr.expression.getText()}"`,
    apply: (arg) => {
        const t0 = hrtime_now_1.now();
        const originalKind = arg.simpleNode.getKind();
        if (!ts_simple_ast_1.TypeGuards.isNewExpression(arg.simpleNode)) {
            arg.simpleNode = arg.simpleNode.getFirstAncestorByKind(ts.SyntaxKind.NewExpression);
        }
        if (!arg.simpleNode || !ts_simple_ast_1.TypeGuards.isNewExpression(arg.simpleNode)) {
            arg.log(`apply fail because couldnt find a NewExpression from node returned by sourcefile..getDescendantAtPos(positionOrRangeToNumber(positionOrRange) - returned node kind was ${typescript_ast_util_1.getKindName(originalKind)}`);
            return;
        }
        const node = arg.simpleNode;
        const argTypes = hrtime_now_1.fromNow(() => node.getArguments().map(arg => arg.getType().getBaseTypeOfLiteralType().getText()), t => arg.log(`argTypes took ${t}`));
        const classDeclaration = hrtime_now_1.fromNow(() => ts_simple_ast_1.TypeGuards.isNewExpression(node) && node.getExpression().getSymbol().getDeclarations()[0], // TODO: check if empty. log and investigate if many 
        // TODO: check if empty. log and investigate if many 
        t => `classDeclaration took ${t}`);
        if (ts_simple_ast_1.TypeGuards.isClassDeclaration(classDeclaration)) {
            const constructorDeclaration = hrtime_now_1.fromNow(() => classDeclaration.addConstructor({
                docs: [],
                scope: ts_simple_ast_1.Scope.Public,
                parameters: argTypes.map((type, i) => ({
                    name: `a${type.substring(0, 1).toUpperCase()}${type.substring(1, type.length)}${i}`,
                    hasQuestionToken: false,
                    type,
                    isRestParameter: false,
                })),
                bodyText: `throw new Error('Not implemented');`
            }), t => arg.log(`addConstructor took ${t}`));
            // heads up : must find openBrace to get position where to insert because constructorDeclaration.getStart() is wrong, see below - ts-simple-ast issue
            const openBrace = classDeclaration.getChildren().find(c => c.getKind() === ts.SyntaxKind.OpenBraceToken);
            return util_1.buildRefactorEditInfo(arg.sourceFile, '\n' + constructorDeclaration.getText(), openBrace.getEnd(), 0);
        }
        else {
            arg.log(`apply fail because node is not ClassDeclaration is ${typescript_ast_util_1.getKindName(classDeclaration.getKind())}`);
        }
        arg.log(`apply took ${hrtime_now_1.timeFrom(t0)}`);
    }
};
// TODO : issue ts-simple-ast  
// const constructorDeclaration = 
//         classDeclaration.addConstructor({
//           docs: ['TODO: Document this autogenerated constructor'], //TODO: configurable
//           scope: Scope.Public,
//           parameters: argTypes.map((type, i) => ({
//             name: `a${type.substring(0, 1).toUpperCase()}${type.substring(1, type.length)}${i}`,
//             hasQuestionToken: false,
//             type,
//             isRestParameter: false,
//           })),
//           bodyText: `throw new Error('Not implemented');`
//         })
//         constructorDeclaration.getStart() is wrong
//# sourceMappingURL=declareConstructor.js.map