import { CodeFix } from '../codeFixes';
/**

# description

declares missing member in a property access expression in some interface or class that the accessed reference implements/extends.

Note: TypeScript 3.0 has some declare method and declare property refactors, but it won't declare in object literals

# attacks
```
    "code": "2339", "message": "Property 'bar' does not exist on type '{ foo: () => number; }'.",
```

# Example:
```
const o = {
  foo: () => { return 1 }
}
const val: string[] = o.bar(1, ['w'], true)      // <---- will add bar as method of literal object o
interface Hello {}
const hello: Hello = {}
let i: string[]
i = hello.world([[1, 2, 3], [4, 5]])             // <----- will add world as method of interface Hello
const k = hello.mama(1, 2, 3) + ' how are you?'  // will add method mama to interface hello
function f(h: Hello) {
  h.fromFunc = true                              // will add boolean property fromFunc to interface hello
}
var x: Date = new Date(hello.timeWhen('born'))    // will add method timeWhen to interface Hello
class C {
  hello: Hello
  m(s: number[]) { this.hello.grasp(s, [false, true]) }  // will add method grasp to interface Hello
}
const notDefined:C
const a = notDefined.foof + 9                              // will add property foof to class C
```

# TODO:

 * use ast structures - we are not considering hasQuestion, modifiers, typeparams, etc
 * Probably we are loosing existing JSdocs ?

 * generate jsdoc for new members added to the interface ?

 * lots of unknown situations - test more

 * declare member in other than interfaces ike classes, literal objects and type declarations: for example this doest work:

```
class C {}
new C().nonExistentMethod()
```

 * (very low priority) return type for method in some scenario

```
interface Hello{}
const hello: Hello = {}
class C {
  hello: Hello
  // here - when we apply refactor on grasp I expect that generated method to return  {modified: Date, fully: boolean} and not just any
  m(s: number[]):{modified: Date, fully: boolean} { return this.hello.grasp(s, [false, true]) }
}
```
*/
export declare const declareMember: CodeFix;
