import { CodeFix } from '../codeFixes';
/**

# Description

declare type for return value but no interface or class declaration is present for that - we create a new interface from return value

# Attacks

"code": "2304",	"message": "Cannot find name 'GResult'.",

# Example,

declare a function like the following - return type is not declared. The suggested fix is to declare it automatically based on the return value of the fn (using typechecking inference):

```
function fn<T>(): FNResult<T> {
  return { a: 1, b: 's', log: (msg) => msg+'', kill: function <T>() { return 1 } }
}
```

# TODO:

 * add jsdoc (configurable ) to new  members  and new interface
 * support unnamed functions and arrow
 * we could offer three alternatives : declare interface, declare type or declare class. see config
 * ISSUE : support spreadproperty assignament and simple property assignment : i.e:  {simple} and {...spread}
 * TODO: members type params (generics) - both in interface and in members and in params.
 * arrow functions with return and without braces _ error!
 * TODO: constructors
 * TODO: getters / setters ?
 * TODO: test jsdoc
 * TODO: check if diagnostic is in the same position  in predicate
 * TODO: appearing in places it shouldnt


 Example showing things that works and things that doesn't:

 ```
 function g(): GResult {	// THIS works
  return { a: 1, b: 's' }
}
const g2 = (): GResult2 => ({ a: 1, b: 's' })// TODO: doesn't work
const g3 = (): GResult3 => { return { a: 1, b: 's' } } // TODO: doesn't work
const h2 = (): HResult2 => ({ // TODO this doesn't work - error!
  a: 1, b: 's', log: (msg) => true, kill: function () { return 1 }
})
const h3 = (): HelloType => {  // TODO this doesn't work
  return {
    a: 1, b: 's', log: (msg) => true, kill: function () { return 1 }
  }
}
function h(): Magallanes { //TODO: generates empty interface
  return {
    a: 1, b: 's', log: (msg: boolean) => { return true }, kill: function () { return 1 }
  }
}
function fn<T>(): FNResult<T> { // THIS works
  return { a: 1, b: 's', log: (msg: string) => { return Math.random() > 0.1 ? true : 'foo' }, kill: function <T>() { return 1 } }
}```

*/
export declare const declareReturnType: CodeFix;
