"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const hrtime_now_1 = require("hrtime-now");
const ts_simple_ast_1 = require("ts-simple-ast");
const ts = __importStar(require("typescript"));
const typescript_ast_util_1 = require("typescript-ast-util");
const util_1 = require("../util");
/**

# Description

declare type for return value but no interface or class declaration is present for that - we create a new interface from return value

# Attacks

"code": "2304",	"message": "Cannot find name 'GResult'.",

# Example,

declare a function like the following - return type is not declared. The suggested fix is to declare it automatically based on the return value of the fn (using typechecking inference):

```
function fn<T>(): FNResult<T> {
  return { a: 1, b: 's', log: (msg) => msg+'', kill: function <T>() { return 1 } }
}
```

# TODO:

 * add jsdoc (configurable ) to new  members  and new interface
 * support unnamed functions and arrow
 * we could offer three alternatives : declare interface, declare type or declare class. see config
 * ISSUE : support spreadproperty assignament and simple property assignment : i.e:  {simple} and {...spread}
 * TODO: members type params (generics) - both in interface and in members and in params.
 * arrow functions with return and without braces _ error!
 * TODO: constructors
 * TODO: getters / setters ?
 * TODO: test jsdoc
 * TODO: check if diagnostic is in the same position  in predicate
 * TODO: appearing in places it shouldnt


 Example showing things that works and things that doesn't:

 ```
 function g(): GResult {	// THIS works
  return { a: 1, b: 's' }
}
const g2 = (): GResult2 => ({ a: 1, b: 's' })// TODO: doesn't work
const g3 = (): GResult3 => { return { a: 1, b: 's' } } // TODO: doesn't work
const h2 = (): HResult2 => ({ // TODO this doesn't work - error!
  a: 1, b: 's', log: (msg) => true, kill: function () { return 1 }
})
const h3 = (): HelloType => {  // TODO this doesn't work
  return {
    a: 1, b: 's', log: (msg) => true, kill: function () { return 1 }
  }
}
function h(): Magallanes { //TODO: generates empty interface
  return {
    a: 1, b: 's', log: (msg: boolean) => { return true }, kill: function () { return 1 }
  }
}
function fn<T>(): FNResult<T> { // THIS works
  return { a: 1, b: 's', log: (msg: string) => { return Math.random() > 0.1 ? true : 'foo' }, kill: function <T>() { return 1 } }
}```

*/
exports.declareReturnType = {
    name: 'declareReturnType',
    config: {
        // TODO: what should we declare. Could be 'interface'|'class'|'type'
        declarationKind: 'interface',
        // TODO: could be true|false|string . add jsdoc to new class/interface declaration
        jsdoc: true,
        // TODO: create interface decl in a separate file ? 
        inNewFile: false
    },
    predicate: (arg) => {
        if (arg.containingTargetLight.kind === ts.SyntaxKind.Identifier &&
            arg.containingTargetLight.parent.kind === ts.SyntaxKind.TypeReference &&
            arg.diagnostics.find(d => d.code === 2304 && d.start === arg.containingTargetLight.getStart())) {
            return true;
        }
        else {
            arg.log('predicate false because child.kind dont match ' + typescript_ast_util_1.getKindName(arg.containingTargetLight.kind));
            return false;
        }
    },
    description: (arg) => `Declare interface "${arg.containingTargetLight.getText()}"`,
    apply: (arg) => {
        const decl = arg.simpleNode.getAncestors().find(ts_simple_ast_1.TypeGuards.isSignaturedDeclaration);
        const interfaceStructure = hrtime_now_1.fromNow(() => inferReturnType(decl, arg), t => arg.log('apply inferReturnType took ' + t));
        const tmpFile = arg.simpleProject.createSourceFile('tmp.ts');
        tmpFile.insertInterface(0, interfaceStructure);
        const code = tmpFile.getText();
        const siblingAncestor = arg.simpleNode.getAncestors().find(a => ts_simple_ast_1.TypeGuards.isSourceFile(a.getParent()));
        return util_1.buildRefactorEditInfo(arg.sourceFile, code, siblingAncestor ? siblingAncestor.getStart() : 0);
    }
};
const inferReturnType = (decl, arg) => {
    const project = arg.simpleProject;
    const tmpFunctionName = `__function_name_${Date.now()}_`;
    const tmpVariableName = `__variable_name_${Date.now()}_`;
    const tmpSourceFile = hrtime_now_1.fromNow(() => project.createSourceFile(`tmp2.ts`, `const ${tmpFunctionName} = ${decl.getText()};  const ${tmpVariableName} = ${tmpFunctionName}()`, { overwrite: true }), (t) => arg.log('apply inferReturnType createSourceFile took ' + t));
    const tmpFuncDecl = tmpSourceFile.getVariableDeclaration(tmpFunctionName).getInitializer();
    if (!ts_simple_ast_1.TypeGuards.isSignaturedDeclaration(tmpFuncDecl)) {
        arg.log(`apply aborted because !TypeGuards.isSignaturedDeclaration(tmpFuncDecl)`);
        return;
    }
    const typeargs = tmpFuncDecl.getReturnType().getTypeArguments();
    hrtime_now_1.fromNow(() => tmpFuncDecl.removeReturnType(), t => arg.log(`apply inferReturnType tmpDecl.removeReturnType() took ${t}`));
    const tmpVarDecl = tmpSourceFile.getVariableDeclaration(tmpVariableName);
    if (!tmpVarDecl) {
        arg.log(`apply aborted because !tmpVarDecl`);
        return;
    }
    const type = hrtime_now_1.fromNow(() => project.getTypeChecker().getTypeAtLocation(tmpVarDecl), t => arg.log(`apply inferReturnType getTypeChecker().getTypeAtLocation took ${t}`));
    const intStructureT0 = hrtime_now_1.now();
    const intStructure = {
        // docs: ['TODO: Document me'],
        name: decl.getReturnTypeNode().getText(),
        properties: type.getProperties()
            .filter(p => {
            const v = p.getValueDeclaration();
            return ts_simple_ast_1.TypeGuards.isPropertyAssignment(v) && !v.getInitializer().getKindName().includes('Function');
        })
            .map(p => {
            const v = p.getValueDeclaration();
            return {
                // docs: ['TODO: Document me'],
                name: p.getName(),
                type: project.getTypeChecker().getTypeAtLocation(p.getValueDeclaration()).getText(),
                val: p.getValueDeclaration(),
                hasQuestionToken: ts_simple_ast_1.TypeGuards.isPropertyAssignment(v) && v.hasQuestionToken(),
            };
        }),
        methods: type.getProperties()
            .filter(p => {
            const v = p.getValueDeclaration();
            return ts_simple_ast_1.TypeGuards.isPropertyAssignment(v) && v.getInitializer().getKindName().includes('Function');
        })
            .map(p => {
            const v = p.getValueDeclaration();
            if (!ts_simple_ast_1.TypeGuards.isPropertyAssignment(v)) {
                return; // TODO: ignoring spread property assignament and simple property assignment : i.e:  {simple} and {...spread}
                // TODO: LOG
            }
            const init = v.getInitializer();
            if (!ts_simple_ast_1.TypeGuards.isArrowFunction(init) && !ts_simple_ast_1.TypeGuards.isFunctionExpression(init)) {
                return;
            }
            return {
                // docs: ['TODO: Document me'],
                name: p.getName(),
                returnType: init.getReturnType() ? init.getReturnType().getText() : 'any',
                parameters: init.getParameters().map(pa => ({
                    name: pa.getName(),
                    type: pa.getType().isTypeParameter() ? pa.getTypeNode().getText() : pa.getType().getText(),
                    hasQuestionToken: pa.hasInitializer() || pa.hasQuestionToken(),
                })),
                typeParameters: init.getTypeParameters().map(p => ({
                    name: p.getName(),
                    constrain: p.getConstraint() && p.getConstraint().getText()
                })),
            };
        })
            .filter(p => p !== undefined),
        typeParameters: typeargs.map(ta => ({ name: ta.getSymbol().getName() })),
    };
    hrtime_now_1.fromNow(() => tmpSourceFile.delete(), t => arg.log(`apply inferReturnType tmpSourceFile.delete() took ${t}`));
    arg.log(`apply inferReturnType create InterfaceStructure took ${hrtime_now_1.timeFrom(intStructureT0)}`);
    return intStructure;
};
//# sourceMappingURL=declareReturnType.js.map