import { CodeFix } from '../codeFixes';
/**
# Description

Extract current class declaration or object literal expression to a new interface and , if possible, make the original declaration to implement it.
```
class Fruit {
  color: string
  private creationDate: Date
  foo() {
    return Promise.resolve([['haha']])
  }
}```

will result in

```
interface IFruit {
  color: string;
  foo(): () => Promise<string[][]>;
  bar(greet: string): Promise<string[]>;
}

class Fruit implements IFruit {
  color: string
  private creationDate: Date
  foo() {
    return Promise.resolve([['haha']])
  }
}```


While in the case of a object literal , there is less metainformation to inffer from :
```
const obj21 = {
  color: 'hshs',
  creationDate: new Date(),
  foo: () => {
    return Promise.resolve([new Date()])
  },
  bar(greet: string): Promise<string[]> { return null }
}
```

will result in
```
interface Iobj21 {
  color: string;
  creationDate: Date;
  foo(): Promise<Date[]>;
  bar(greet: string): Promise<string[]>;
}
const obj21: Iobj21 = {
  color: 'hshs',
  creationDate: new Date(),
  foo: () => {
    console.log('hello');
    return Promise.resolve([new Date()])
  },
  bar(greet: string): Promise<string[]> { return null }
}
```

# TODO

 * config
 * if the class already implements an interface, we shouldn't extract those methods.
 * what about methods of super classes ? this behavior could be configurable
 * put the interface in a separate file and add an import?
 * enhancement: if user selects only some methods, we should only extract those.
 * properties declared as constructor scoped params

*/
export declare const extractInterface: CodeFix;
