"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ts_simple_ast_1 = require("ts-simple-ast");
const ts = __importStar(require("typescript"));
const typescript_ast_util_1 = require("typescript-ast-util");
const util_1 = require("../util");
/**

# Description:

When implementation miss to implement a member or is doing it incorrectly, select member name and accept suggestion "implement interface"

# Diagnostic error attacked:

"code": "2416","message": "Property 'method1' in type 'SomeImplementation' is not assignable to the same property in base type 'SomeInterface'.\n  Type '(param: number) => number[]' is not assignable to type '(param: string) => number[]'.\n    Types of parameters 'param' and 'param' are incompatible.\n      Type 'string' is not assignable to type 'number'.",

# How to reproduce:
```
interface SomeInterface {
  prop1: { s: string, n: Date }[]
}
class Class2 implements SomeInterface {
  prop1: boolean[]        <----- here select prop1 and implement interface it will fix prop1 signature
}
```

# TODO

 * TODO: work for constructors and setter/getter members
 * TODO: review
 *
 * TODO: this is failing:
class Unit {
  energy: number
}
const unit: Unit = {
  energy: 123, color: 'red'
}

 *  we choose any member signature - we should choose the most similar one
 
 * config:
```
config: {
    // recursive tre will generate the whole sub literals..
    recursive: false,
    // add missing properties declared to interface so there are no errors in strict mode
    addMissingPropertiesToInterface: false
  },
```
*/
exports.implementInterfaceMember = {
    name: 'implementInterfaceMember',
    config: {
        // recursive tre will generate the whole sub literals.. 
        recursive: false,
        // add missing properties declared to interface so there are no errors in strict mode
        addMissingPropertiesToInterface: false
    },
    predicate: (arg) => {
        if (arg.containingTargetLight.kind === ts.SyntaxKind.Identifier &&
            arg.diagnostics.find(d => d.code === 2416 && d.start === arg.containingTargetLight.getStart())) {
            return true;
        }
        else {
            arg.log('predicate false because child.kind dont match ' + typescript_ast_util_1.getKindName(arg.containingTargetLight.kind));
            return false;
        }
    },
    description: (arg) => `Implement interface`,
    apply: (arg) => {
        const id = arg.simpleNode;
        const member = id.getParent();
        const decl = member.getParent();
        if (!(ts_simple_ast_1.TypeGuards.isIdentifier(id) &&
            (ts_simple_ast_1.TypeGuards.isMethodDeclaration(member) || ts_simple_ast_1.TypeGuards.isPropertyDeclaration(member)) &&
            ts_simple_ast_1.TypeGuards.isClassDeclaration(decl))) {
            return arg.log(`predicate not complied: decl: ${decl.getKindName()} member: ${member.getKindName()} id: ${id.getKindName()}`);
        }
        const interfaceWithMemberName = util_1.findInterfacesWithPropertyNamed(decl, id.getText()).pop(); // TODO: we choose any member signature - we should choose the most similar one
        const memberSignature = interfaceWithMemberName.getMembers().filter(ts_simple_ast_1.TypeGuards.isPropertyNamedNode).pop(); // TODO: any arbitrary signature
        if (ts_simple_ast_1.TypeGuards.isMethodSignature(memberSignature) && ts_simple_ast_1.TypeGuards.isMethodDeclaration(member)) {
            util_1.fixSignature(member, memberSignature);
        }
        else if (ts_simple_ast_1.TypeGuards.isPropertySignature(memberSignature) && ts_simple_ast_1.TypeGuards.isPropertyDeclaration(member)) {
            member.setType(memberSignature.getType().getText());
        }
    }
};
//# sourceMappingURL=implementInterfaceMember.js.map