import { CodeFix } from '../codeFixes';
/**
 
# Description

Change class member to public when user access a private member

# Attacks
```
"code": "2341",
"message": "Property 'method' is private and only accessible within class 'A'.",

or

"code": "2673",
"message": "Constructor of class 'A' is private and only accessible within the class declaration.",
  

# Example

```
class A {
  private method(a: number):Date[]{
    return [new Date()]
  }
}
const a = new A()
new A().method(1) // suggestion
a.method(2) // suggestion
```

in both suggestions the resulting code will be

```
class A {
  public method(a: number):Date[]{
    return [new Date()]
  }
}
....

# TODO

 * test get/set property accessor
 * test with constructors

*/
export declare const memberChangeScope: CodeFix;
