"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ts_simple_ast_1 = require("ts-simple-ast");
const ts = __importStar(require("typescript"));
const typescript_ast_util_1 = require("typescript-ast-util");
let targetNode;
/**
 
# Description

Change class member to public when user access a private member

# Attacks
```
"code": "2341",
"message": "Property 'method' is private and only accessible within class 'A'.",

or

"code": "2673",
"message": "Constructor of class 'A' is private and only accessible within the class declaration.",
  

# Example

```
class A {
  private method(a: number):Date[]{
    return [new Date()]
  }
}
const a = new A()
new A().method(1) // suggestion
a.method(2) // suggestion
```

in both suggestions the resulting code will be

```
class A {
  public method(a: number):Date[]{
    return [new Date()]
  }
}
....

# TODO

 * test get/set property accessor
 * test with constructors

*/
exports.memberChangeScope = {
    name: 'memberChangeScope',
    config: {},
    predicate: (options) => {
        if (((targetNode = options.containingTargetLight.kind === ts.SyntaxKind.Identifier ? options.containingTargetLight : undefined) ||
            (targetNode = typescript_ast_util_1.findAscendant(options.containingTargetLight, ts.isNewExpression, true))) &&
            options.diagnostics.find(d => (d.code === 2341 || d.code === 2673) &&
                d.start <= options.containingTargetLight.getStart() &&
                d.start + d.length >= options.containingTargetLight.getEnd())) {
            return true;
        }
        else {
            options.log('predicate false because d.code === 2341 || d.code === 2673 || (Identifier||isNewExpression) != ' + typescript_ast_util_1.getKindName(options.containingTargetLight.kind));
            return false;
        }
    },
    description: (options) => {
        let what = ts.isIdentifier(targetNode) ? `member ${options.containingTargetLight.getText()}` : ts.isNewExpression(targetNode) ? `constructor ${targetNode.expression.getText()}` : targetNode ? targetNode.getText() : options.containingTargetLight.getText();
        return `Make ${what} public`;
    },
    apply: (options) => {
        const parent = options.simpleNode.getParent();
        let name;
        if (ts_simple_ast_1.TypeGuards.isPropertyAccessExpression(parent)) {
            name = parent.getNameNode();
        }
        else if (ts_simple_ast_1.TypeGuards.isNewExpression(parent)) {
            name = parent.getExpression();
        }
        else {
            options.log(`apply false because ${parent.getKindName()} !== !TypeGuards.isPropertyAccessExpression(parent) || !TypeGuards.isNewExpression(parent)`);
            return;
        }
        const declarations = name.getSymbol().getDeclarations();
        if (!declarations || declarations.length === 0) {
            options.log(`scope not changed because !declarations || declarations.length===0`);
            return;
        }
        declarations.map(declaration => {
            if (ts_simple_ast_1.TypeGuards.isClassDeclaration(declaration)) {
                declaration.getConstructors()[0].setScope(ts_simple_ast_1.Scope.Public); //TODO: select the one that matches the new expr call 
                options.log(`constructor scope changed to public`);
            }
            else if (ts_simple_ast_1.TypeGuards.isScopedNode(declaration)) {
                declaration.setScope(ts_simple_ast_1.Scope.Public);
                options.log(`scope changed to public`);
            }
            else {
                options.log(`scope not changed because ${declaration.getKindName()} not isScopedNode nor isClassDeclaration `);
            }
        });
    }
};
//# sourceMappingURL=memberChangeScope.js.map