import * as ts from 'typescript';
import { CodeFix, CodeFixOptions } from "../codeFixes";
/**

# Description

From anywhere inside a class' property declaration or an interface's property signature user will be suggested with the refactor "Delegate methods in property "foo". If executed it will create the same methods of property foo type delegating the implementation on them

# Example
```
interface Speedometer {
  getCurrentSpeed(): number
  rotate(force: number): { counterclockwise: boolean, h: number }
  m(): number
  go(to: { x: number, y: number }): Promise<void>
}
class Fiat1 {
  speedometer: Speedometer // suggested from anywhere inside the property declaration
}
```
the result of accepting this refactor standing on property `Fiat1.speedometer` is:

```
class Fiat1 {
  speedometer: Speedometer
  public getCurrentSpeed(): number {
    return this.speedometer.getCurrentSpeed();
  }
  public rotate(force: number): { counterclockwise: boolean, h: number } {
    return this.speedometer.rotate(force);
  }
  public m(): number {
    return this.speedometer.m();
  }
  public go(to: { x: number, y: number }): Promise<void> {
    return this.speedometer.go(to);
  }
}
```

# TODO

 * what happen when property type is a class ? in that case what happens with non method members like properties and getter/setters ?
 * undefined check for properties that can be undefined
 * config
 * check these comments: TODO: we do only first level - by configuration we could do it recursively
 * support object literal expressions ?

*/
export declare class MethodDelegate implements CodeFix {
    name: string;
    config: {
        delegateOnlyOnNonUndefinedProperties: boolean;
        checkForUndefinedInDelegationImplementation: boolean;
    };
    propertyDeclaration: ts.PropertyDeclaration;
    predicate(arg: CodeFixOptions): boolean;
    description(arg: CodeFixOptions): string;
    apply(arg: CodeFixOptions): void;
}
export declare const methodDelegate: MethodDelegate;
