"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_ast_util_1 = require("typescript-ast-util");
const ts = __importStar(require("typescript"));
const toNamedParams_1 = require("./toNamedParams");
/**
# Description

This will remove comments selection that don't match with configurable patterns , by default those line comments that doesn't start with "TODO" or "heads up"

*/
class RemoveComments {
    constructor() {
        this.name = 'removeComments';
        this.config = {
            commentType: 'line',
            pattern: (s) => !s.toLowerCase().trim().startsWith('todo') || !s.toLowerCase().trim().startsWith('heads up')
        };
    }
    predicate(options) {
        if (!options.positionOrRange || options.positionOrRange.pos === undefined) {
            options.log('predicate false because not a range : ' + JSON.stringify(options.positionOrRange));
            return false;
        }
        const range = options.positionOrRange;
        const lineAndChar1 = options.sourceFile.getLineAndCharacterOfPosition(range.pos);
        const lineAndChar2 = options.sourceFile.getLineAndCharacterOfPosition(range.end);
        if (lineAndChar2.line - lineAndChar1.line > 1) {
            return true;
        }
        else {
            options.log('predicate false because not enough distance between ' + JSON.stringify(lineAndChar1) + ' and ' + JSON.stringify(lineAndChar2));
            return false;
        }
    }
    description(options) { return `Remove comments`; }
    apply(options) {
        if (!options.positionOrRange || options.positionOrRange.pos === undefined) {
            return;
        }
        const range = options.positionOrRange;
        const sourceFile = options.simpleNode.getSourceFile();
        let textChanges = [];
        sourceFile.getDescendants()
            .forEach(d => {
            d.getLeadingCommentRanges().concat(d.getTrailingCommentRanges())
                .forEach(c => {
                const match = c.getKind() === ts.SyntaxKind.SingleLineCommentTrivia && c.getPos() >= range.pos && c.getEnd() <= range.end && this.config.pattern(c.getText());
                if (match) {
                    textChanges.push(toNamedParams_1.buildTextChange('', c.getPos(), c.getWidth()));
                }
            });
        });
        textChanges = typescript_ast_util_1.unionEquals(textChanges, [], (a, b) => JSON.stringify(a) === JSON.stringify(b));
        const refactor = {
            edits: [
                {
                    fileName: sourceFile.getFilePath(),
                    textChanges,
                }
            ]
        };
        return refactor;
    }
}
exports.RemoveComments = RemoveComments;
exports.removeComments = new RemoveComments();
//# sourceMappingURL=removeComments.js.map