"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ts = __importStar(require("typescript"));
const typescript_ast_util_1 = require("typescript-ast-util");
const ts_simple_ast_1 = require("ts-simple-ast");
const util_1 = require("../util");
/**

# Description

This refactor will split a variable declaration list with multiple variable declarations into single variable declaration statements.

# Example

```let i = 0, c = 's', arr = []```

Selecting any fragment of previous statement will suggest te refactor "Split variable declaration list" and if applied it will result in the following code:

```
let i = 0
let c = 's'
let arr = []
```

It doesn't attack any error diagnostic.

# TODO
 * unsplit variables - if we are in a more than one var decl statement
 * config
*/
exports.splitVariableDeclarationList = {
    name: 'splitVariableDeclarationList',
    config: {
        // TODO; add inferred types to generated var decl stmts ?  currently adding. 
        addTypes: true
    },
    predicate: (arg) => {
        const varDeclList = typescript_ast_util_1.findAscendant(arg.containingTargetLight, ts.isVariableDeclarationList, true);
        if (!varDeclList) {
            arg.log('predicate false because no VariableDeclarationList ascendant found - containingTarget was ' + typescript_ast_util_1.getKindName(arg.containingTargetLight));
            return false;
        }
        else if (varDeclList.declarations.length <= 1) {
            arg.log('predicate false because list has only one declaration');
            return false;
        }
        return true;
    },
    description: (arg) => `Split variable declaration list`,
    apply: (arg) => {
        const varDeclList = arg.simpleNode.getFirstAncestorByKind(ts.SyntaxKind.VariableDeclarationList);
        if (!varDeclList || !ts_simple_ast_1.TypeGuards.isVariableDeclarationList(varDeclList)) {
            arg.log('not apply because no splitVariableDeclarationList ancestor was found');
            return;
        }
        const container = varDeclList.getParent().getParent();
        if (ts_simple_ast_1.TypeGuards.isBlock(container) || ts_simple_ast_1.TypeGuards.isSourceFile(container)) {
            const variableStatements = varDeclList.getDeclarations().map(d => ({
                declarationKind: varDeclList.getDeclarationKindKeyword().getText(),
                declarations: [{
                        hasExclamationToken: d.hasExclamationToken(),
                        name: d.getName(),
                        type: d.getTypeNode() ? d.getTypeNode().getText() : d.getType().getText(),
                        initializer: d.getInitializer().getText()
                    }]
            }));
            const variableStatement = arg.simpleNode.getFirstAncestorByKind(ts.SyntaxKind.VariableStatement);
            const start = variableStatement.getStart();
            const length = variableStatement.getFullWidth(); // + 2 // TODO: don't know why I have to do +2 in order to work OK
            const variableStatementNodes = container.insertVariableStatements(0, variableStatements);
            const code = variableStatementNodes.map(s => s.getText()).join('\n'); // TODO: respect formatOptions
            return util_1.buildRefactorEditInfo(arg.sourceFile, code, start, length);
        }
        else {
            arg.log(`not apply because condition not meet: TypeGuards.isBlock(container) || TypeGuards.isSourceFile(container) === ${ts_simple_ast_1.TypeGuards.isBlock(container) || ts_simple_ast_1.TypeGuards.isSourceFile(container)} - container kind is ${container.getKindName()}`);
        }
    }
};
//# sourceMappingURL=splitVariableDeclarationList.js.map