"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ts_simple_ast_1 = require("ts-simple-ast");
const ts = __importStar(require("typescript"));
const typescript_ast_util_1 = require("typescript-ast-util");
const tsserverlibrary_1 = require("typescript/lib/tsserverlibrary");
/**
 
# Description

see https://github.com/Microsoft/TypeScript/issues/23552

# Attacks

This refactor doesn't attack any diagnostic error

# Example

```
function foo (a: number, b: string[], c: (n:number)=>boolean[], d?: boolean){}
```

select a fragment of the parameter list will suggest "declare interface from named parameters" and if applied will change the code to :

```
interface FooOptions{
  a: number
  b: string[]
  c?: boolean
}
function foo({a,b,c}: FooOptions){}
```

# TODO

*/
exports.toNamedParameters = {
    name: 'toNamedParameters',
    config: {
        // TODO: what should we declare. Could be 'interface'|'class'|'type'
        declarationKind: 'interface',
        // TODO: could be true|false|string . add jsdoc to new class/interface declaration
        jsdoc: true,
        // TODO: create interface decl in a separate file ? 
        inNewFile: false
    },
    predicate: (arg) => {
        const param = typescript_ast_util_1.findAscendant(arg.containingTargetLight, ts.isParameter, true);
        if (!param) {
            arg.log('predicate false because no  Parameter ascendant found - containingTarget was ' +
                typescript_ast_util_1.getKindName(arg.containingTargetLight));
            return false;
        }
        return true;
    },
    description: (arg) => `Convert to Named Parameters`,
    apply: (arg) => {
        const functionLikeDeclaration = arg.simpleNode.getAncestors().find(ts_simple_ast_1.TypeGuards.isFunctionLikeDeclaration);
        if (!functionLikeDeclaration) {
            arg.log('not applied because FunctionLikeDeclaration ancestor not found');
            return;
        }
        const container = functionLikeDeclaration.getParent();
        if (!ts_simple_ast_1.TypeGuards.isStatementedNode(container)) {
            arg.log('FunctionLikeDeclaration.getParent not applied because is not TypeGuards.isStatementedNode(container)');
            return;
        }
        let interfaceName = functionLikeDeclaration.getName() ? functionLikeDeclaration.getName() + '' : 'UnnamedOptions';
        interfaceName = interfaceName.substring(0, 1).toUpperCase() + interfaceName.substring(1, interfaceName.length);
        const interfaceStructure = {
            docs: ['TODO: Document me'],
            typeParameters: functionLikeDeclaration.getTypeParameters().map(tp => ({
                name: tp.getName(),
                constrain: tp.getConstraint() && tp.getConstraint().getText()
            })),
            name: interfaceName,
            properties: functionLikeDeclaration.getParameters().filter(param => !param.getTypeNode() || param.getTypeNode().getKind() !== ts.SyntaxKind.FunctionType) // TODO: this will ignore/wrong-print parameters of type function that have an assignament
                .map(property => ({
                // TODO :reuse function-like @param jsdoc commend or create a new jsdoc comment if user configured to do so.
                name: property.getName(),
                type: property.getTypeNode() && property.getTypeNode().getText() || property.getType() && property.getType().getText(),
                hasQuestionToken: property.hasQuestionToken()
            })),
            methods: functionLikeDeclaration.getParameters().filter(param => param.getTypeNode() && param.getTypeNode().getKind() == ts.SyntaxKind.FunctionType).map(method => ({
                // TODO : reuse function-like @param jsdoc commend or create a new jsdoc comment if user configured to do so.
                name: method.getName(),
                returnType: method.getTypeNode() && method.getTypeNode().getReturnTypeNode() && method.getTypeNode().getReturnTypeNode().getText(),
                parameters: method.getTypeNode().getParameters().map(param => ({
                    name: param.getName(),
                    type: param.getTypeNode() && param.getTypeNode().getText() || param.getType() && param.getType().getText(),
                    hasQuestionToken: param.hasQuestionToken()
                })),
                hasQuestionToken: method.hasQuestionToken()
            })),
        };
        const textChanges = [];
        const dummySourceFile = arg.simpleProject.createSourceFile('tmp12.ts');
        const intDecl = dummySourceFile.insertInterface(0, interfaceStructure);
        textChanges.push(buildTextChange('\n' + intDecl.getFullText() + '\n', functionLikeDeclaration.getFullStart()));
        const parameterDeclarationStructure = {
            name: '{' + functionLikeDeclaration.getParameters().map(param => param.getName() + (param.getInitializer() ? (' = ' + param.getInitializer().getText()) : '')).join(', ') + '}',
            type: interfaceName + (functionLikeDeclaration.getTypeParameters().length ?
                `<${functionLikeDeclaration.getTypeParameters().map(tp => tp.getText()).join(', ')}>` : ''),
        };
        const list = functionLikeDeclaration.getFirstChildByKind(tsserverlibrary_1.SyntaxKind.SyntaxList);
        const width = list.getFullWidth();
        // textChanges.push(buildTextChange('', list.getFullStart()+1, list.getFullWidth()+1))
        functionLikeDeclaration.getParameters().forEach(param => param.remove());
        const param = functionLikeDeclaration.addParameter(parameterDeclarationStructure);
        textChanges.push(buildTextChange(param.getText(), param.getFullStart(), width));
        const refactor = {
            edits: [
                {
                    fileName: container.getSourceFile().getFilePath(),
                    textChanges
                }
            ]
        };
        return refactor;
    }
};
function buildTextChange(newText, start, length = 0) {
    return {
        newText,
        span: { start, length }
    };
}
exports.buildTextChange = buildTextChange;
//# sourceMappingURL=toNamedParams.js.map